Package: pROC
Type: Package
Title: Display and Analyze ROC Curves
Version: 1.19.0.1
Date: 2025-07-31
Encoding: UTF-8
Depends: R (>= 2.14)
Imports: methods, Rcpp (>= 0.11.1)
Suggests: tcltk, MASS, logcondens, testthat, vdiffr, ggplot2, rlang
LinkingTo: Rcpp
Authors@R: c(person("Xavier", "Robin", role = c("cre", "aut"), 
                    email = "pROC-cran@xavier.robin.name",
                    comment = c(ORCID = "0000-0002-6813-3200")),
             person("Natacha", "Turck", role = "aut"),
             person("Alexandre", "Hainard", role = "aut"),
             person("Natalia", "Tiberti", role = "aut"),
             person("Frédérique", "Lisacek", role = "aut"),
             person("Jean-Charles", "Sanchez", role = "aut"),
             person("Markus", "Müller", role = "aut"),
             person("Stefan", "Siegert", role = "ctb", 
                    comment = "Fast DeLong code", 
                    email = "stefan_siegert@gmx.de"),
             person("Matthias", "Doering", role = "ctb", 
                    comment = "Hand & Till Multiclass"),
             person("Zane", "Billings", role = "ctb", 
                    comment = "DeLong paired test CI"))
Description: Tools for visualizing, smoothing and comparing receiver operating characteristic (ROC curves). (Partial) area under the curve (AUC) can be compared with statistical tests based on U-statistics or bootstrap. Confidence intervals can be computed for (p)AUC or ROC curves.
License: GPL (>= 3)
URL: https://xrobin.github.io/pROC/
BugReports: https://github.com/xrobin/pROC/issues
LazyData: yes
NeedsCompilation: yes
Packaged: 2025-07-31 07:57:35 UTC; xavier
Author: Xavier Robin [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-6813-3200>),
  Natacha Turck [aut],
  Alexandre Hainard [aut],
  Natalia Tiberti [aut],
  Frédérique Lisacek [aut],
  Jean-Charles Sanchez [aut],
  Markus Müller [aut],
  Stefan Siegert [ctb] (Fast DeLong code),
  Matthias Doering [ctb] (Hand & Till Multiclass),
  Zane Billings [ctb] (DeLong paired test CI)
Maintainer: Xavier Robin <pROC-cran@xavier.robin.name>
Repository: CRAN
Date/Publication: 2025-07-31 10:00:14 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 15:58:34 UTC; unix
Archs: pROC.so.dSYM
