export(are.paired)
S3method("are.paired", "auc")
S3method("are.paired", "roc")
S3method("are.paired", "smooth.roc")

export(auc)
S3method("auc", "roc")
S3method("auc", "smooth.roc")
S3method("auc", "formula")
S3method("auc", "default")
S3method("auc", "multiclass.roc")
S3method("auc", "mv.multiclass.roc")

export(ci)
S3method("ci", "roc")
S3method("ci", "smooth.roc")
S3method("ci", "default")
S3method("ci", "formula")
S3method("ci", "auc")
S3method("ci", "multiclass.roc")
S3method("ci", "multiclass.auc")

export(ci.coords)
S3method("ci.coords", "roc")
S3method("ci.coords", "smooth.roc")
S3method("ci.coords", "default")
S3method("ci.coords", "formula")

export(ci.thresholds)
S3method("ci.thresholds", "roc")
S3method("ci.thresholds", "smooth.roc")
S3method("ci.thresholds", "default")
S3method("ci.thresholds", "formula")

export(ci.sp)
S3method("ci.sp", "roc")
S3method("ci.sp", "smooth.roc")
S3method("ci.sp", "default")
S3method("ci.sp", "formula")

export(ci.se)
S3method("ci.se", "roc")
S3method("ci.se", "smooth.roc")
S3method("ci.se", "default")
S3method("ci.se", "formula")

export(ci.auc)
S3method("ci.auc", "roc")
S3method("ci.auc", "smooth.roc")
S3method("ci.auc", "default")
S3method("ci.auc", "formula")
S3method("ci.auc", "auc")
S3method("ci.auc", "multiclass.roc")
S3method("ci.auc", "multiclass.auc")

export(coords)
S3method("coords", "auc")
S3method("coords", "roc")
S3method("coords", "smooth.roc")

export(cov)
S3method("cov", "roc")
S3method("cov", "smooth.roc")
S3method("cov", "default")
S3method("cov", "auc")

export(has.partial.auc)
S3method("has.partial.auc", "roc")
S3method("has.partial.auc", "smooth.roc")
S3method("has.partial.auc", "auc")

export(multiclass.roc)
S3method("multiclass.roc", "default")
S3method("multiclass.roc", "formula")

export(power.roc.test)
S3method("power.roc.test", "roc")
S3method("power.roc.test", "numeric")
S3method("power.roc.test", "list")

S3method("print", "roc")
S3method("print", "smooth.roc")
S3method("print", "auc")
S3method("print", "ci.auc")
S3method("print", "ci.thresholds")
S3method("print", "ci.se")
S3method("print", "ci.sp")
S3method("print", "ci.coords")
S3method("print", "multiclass.roc")
S3method("print", "multiclass.auc")
S3method("print", "mv.multiclass.roc")
S3method("print", "mv.multiclass.auc")

export(roc)
S3method("roc", "default")
S3method("roc", "formula")
S3method("roc", "data.frame")

export(roc_)

export(roc.test)
S3method("roc.test", "roc")
S3method("roc.test", "smooth.roc")
S3method("roc.test", "default")
S3method("roc.test", "formula")
S3method("roc.test", "auc")

export(smooth)
S3method("smooth", "roc")
S3method("smooth", "smooth.roc")
S3method("smooth", "default")

export(var)
S3method("var", "roc")
S3method("var", "smooth.roc")
S3method("var", "default")
S3method("var", "auc")

S3method("Ops", "auc")
S3method("Ops", "ci.se")
S3method("Ops", "ci.sp")
S3method("Ops", "ci.auc")

S3method("Math", "auc")
S3method("Math", "ci.se")
S3method("Math", "ci.sp")
S3method("Math", "ci.auc")

S3method("lines", "roc")
S3method("lines", "smooth.roc")

export(lines.roc)
S3method("lines.roc", "roc")
S3method("lines.roc", "smooth.roc")
S3method("lines.roc", "formula")
S3method("lines.roc", "default")

S3method("plot", "roc")
S3method("plot", "smooth.roc")
S3method("plot", "ci.thresholds")
S3method("plot", "ci.sp")
S3method("plot", "ci.se")
S3method("plot", "ci.coords")

export(plot.roc)
S3method("plot.roc", "roc")
S3method("plot.roc", "smooth.roc")
S3method("plot.roc", "formula")
S3method("plot.roc", "default")

export(ggroc)
S3method("ggroc", "roc")
S3method("ggroc", "smooth.roc")
S3method("ggroc", "list")

export(geom_polygon_auc)
S3method("geom_polygon_auc", "auc")
S3method("geom_polygon_auc", "roc")
S3method("geom_polygon_auc", "smooth.roc")

#export(select)
#export(select_)
#importFrom("dplyr", "select")
#importFrom("dplyr", "select_")
#S3method("select", "roc")
#S3method("select_", "roc")

# Fix R CMD check warning false positives
# "apparent S3 methods exported but not registered"
# Note: these methods have an export() above
S3method("roc", "test")
S3method("ci", "coords")
S3method("ci", "se")
S3method("ci", "sp")
S3method("ci", "thresholds")

import(Rcpp, grDevices, graphics, stats)
useDynLib(pROC, .registration = TRUE)
