# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

RcppVersion <- function() {
    .Call(`_pROC_RcppVersion`)
}

delongPlacementsCpp <- function(roc) {
    .Call(`_pROC_delongPlacementsCpp`, roc)
}

rocUtilsPerfsAllC <- function(thresholds, controls, cases, direction) {
    .Call(`_pROC_rocUtilsPerfsAllC`, thresholds, controls, cases, direction)
}

